'use strict';

var EventEmitter = require('events').EventEmitter;

var util = require('./util.js');

var create = function(initValues) {
  var events = new EventEmitter();
  var values = initValues || {};

  var add = function(key, value) {
    if (value === undefined) { value = key; }
    if (typeof key !== 'string') { throw new Error('Key must be a string'); }
    if (typeof value !== 'string') { throw new Error('Value must be a string' + JSON.stringify(value)); }
    if (!util.has(values, key)) {
      values[key] = {};
      events.emit('add', key);
      values[key][value] = true;
      return true;
    }
    values[key][value] = true;
    return false;
  };

  var remove = function(key, value) {
    if (value === undefined) { value = key; }

    if (util.has(values, key)) {
      delete values[key][value];
      if (util.isEmpty(values[key])) {
        delete values[key];
        events.emit('remove', key);
        return true;
      }
    }

    return false;
  };

  var removeAll = function(key) {
    if (util.has(values, key)) {
      delete values[key];
      events.emit('remove', key);
      return true;
    }

    return false;
  };

  var invert = function() {
    var inverted = {};
    for(var key in values) {
      if (Object.prototype.hasOwnProperty.call(values, key)) {
        var innerValues = values[key];
        for(var innerValue in innerValues) {
          if (Object.prototype.hasOwnProperty.call(innerValues, innerValue)) {
            if (!Object.prototype.hasOwnProperty.call(inverted, innerValue)) {
              inverted[innerValue] = {};
            }
            inverted[innerValue][key] = true;
          }
        }
      }
    }
    return create(inverted);
  };

  events.values = values;
  events.add = add;
  events.remove = remove;
  events.removeAll = removeAll;
  events.invert = invert;

  return events;
};

module.exports = create;

