'use strict';

var multimap = require('./multimap.js');
var util = require('./util.js');

var enableSupress = function() {
  var values = multimap();
  var supressors = multimap();
  var supressedValues = multimap();

  var oldAdd = values.add;
  var oldRemove = values.remove;
  var oldRemoveAll = values.removeAll;

  var add = function(key, value) {
    if (typeof key !== 'string') { throw new Error('Key must be a string'); }
    if (value !== undefined && typeof value !== 'string') { throw new Error('Value must be a string'); }
    if (util.has(supressors.values, key)) {
      supressedValues.add(key, value);
      return false;
    } else {
      return oldAdd(key, value);
    }
  };

  var remove = function(key, value) {
    if (util.has(supressors.values, key)) {
      supressedValues.remove(key, value);
      return false;
    } else {
      return oldRemove(key, value);
    }
  };

  var removeAll = function(key) {
    if (util.has(supressors.values, key)) {
      supressedValues.removeAll(key);
      return false;
    } else {
      return oldRemoveAll(key);
    }
  };

  var supress = function(key, value) {
    var ret = supressors.add(key, value);
    if (ret && Object.prototype.hasOwnProperty.call(values.values, key)) {
      var oldValues = values.values[key];
      for (var oldValue in oldValues) {
        if (Object.prototype.hasOwnProperty.call(oldValues, oldValue)) {
          supressedValues.add(key, oldValue);
        }
      }
      oldRemoveAll(key);
    }
    return ret;
  };

  var unsupress = function(key, value) {
    var ret = supressors.remove(key, value);
    if (ret && Object.prototype.hasOwnProperty.call(supressedValues.values, key)) {
      var oldValues = supressedValues.values[key];
      for (var oldValue in oldValues) {
        if (Object.prototype.hasOwnProperty.call(oldValues, oldValue)) {
          oldAdd(key, oldValue);
        }
      }
      supressedValues.removeAll(key);
    }
    return ret;
  };

  values.add = add;
  values.remove = remove;
  values.removeAll = removeAll;
  values.supress = supress;
  values.unsupress = unsupress;

  return values;
};

module.exports = enableSupress;
