#!/usr/bin/env node

'use strict';

var _ = require('underscore');
var dg = require('../lib/main.js');

var requirements = dg.multimap();
requirements.add('serverA', 'serverB');
requirements.add('serverA', 'sharedA');
requirements.add('serverB', 'serverC');
requirements.add('serverE', 'serverF');
requirements.add('serverD', 'serverZ');
requirements.add('clientA', 'serverB');
requirements.add('clientA', 'clientB');
requirements.add('clientA', 'serverE');
requirements.add('clientB', 'clientC');
requirements.add('clientC', 'clientD');

var inverse = requirements.invert();

var recommendations = dg.multimap();
recommendations.add('serverC', 'clientA');
recommendations.add('serverC', 'clientB');

var required = dg.supressmap();
var forbidden = dg.supressmap();
var selected = dg.supressmap();

dg.expand('requirements', selected, selected, requirements);
dg.expand('requirements', required, required, requirements);
dg.expand('forbidden', forbidden, forbidden, inverse);
dg.expand('recommendedByRequired', required, selected, recommendations);
dg.expand('recommendedBySelected', selected, selected, recommendations);

dg.exclude('forbidden', forbidden, required);
dg.exclude('forbidden', forbidden, selected);
dg.exclude('required', required, selected);

required.add('serverA');
required.add('serverB');
required.add('clientC');

forbidden.add('serverZ');
forbidden.add('clientZ');

required.on('add', function(val) {
  console.log(val + ' is now required');
});

selected.on('add', function(val) {
  console.log(val + ' is now selected');
});

forbidden.on('add', function(val) {
  console.log(val + ' is now forbidden');
});

required.on('remove', function(val) {
  console.log(val + ' is no longer required');
});

selected.on('remove', function(val) {
  console.log(val + ' is no longer selected');
});

forbidden.on('remove', function(val) {
  console.log(val + ' is no longer forbidden');
});

var print = function() {
  console.log('REQUIRED:');
  console.log(JSON.stringify(_.keys(required.values)));
  console.log();
  console.log('FORBIDDEN:');
  console.log(JSON.stringify(_.keys(forbidden.values)));
  console.log();
  console.log('SELECTED:');
  console.log(JSON.stringify(_.keys(selected.values)));
  console.log();
};

print();

exports.print = print;
exports.required = required;
exports.forbidden = forbidden;
exports.selected = selected;

